.mediaplayer-menu {
  width: 360px;
}

.controls {
    padding-top: 0.4em;
    spacing: 8px;
    padding-bottom: 0.4em;
}

.indicator-icon {
    padding-right: 0px;
}

.indicators {
    spacing: 0px;
}

.indicator {
    padding-right: 1px;
}

.secondary-indicator {
    icon-size: 8px;
    padding-top: 0.5em;
}

.third-indicator {
    icon-size: 8px;
    padding-left: 0.5em;
}

.track-box {
    padding-left: 2.1em;
    padding-right: 2em;
}

.track-cover {
    border: 1px solid rgba(255, 255, 255, 0.5);
    border-radius: 2px;
}

.track-infos:ltr {
    padding-left: 0em;
}

.track-infos:rtl {
    padding-left: 0em;
}

.track-info {
    padding-bottom: 0.2em;
}

.track-info-big {
  font-weight: bold;
  font-size: 1.1em;
}

.track-info-medium {
  font-weight: bold;
}

.track-rating {
    padding-top: 0.2em;
    font-size: 0.9em;
}

.menu-icon {
    icon-size: 1.14em;
}

.slider-item .slider-text {
    font-size: 0.9em;
    width: 7em;
}

.popup-menu-item:active {
    color: white;
}

.system-menu-action:disabled {
    color: #999;
}
